==> quickscale-0.18.0.tar.gz: quickscale-0.18.0/quickscale/templates/credits/tests/test_subscription.py <==
"""Tests for subscription functionality."""
from django.test import TestCase, Client, override_settings
from django.contrib.auth import get_user_model
from django.urls import reverse
from django.utils import timezone
from datetime import timedelta
from decimal import Decimal
from unittest.mock import patch, MagicMock

from credits.models import UserSubscription, CreditAccount, CreditTransaction
from stripe_manager.models import StripeProduct, StripeCustomer

User = get_user_model()


@override_settings(
    DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}},
    STRIPE_ENABLED=False,
    STRIPE_LIVE_MODE=False,
)
class UserSubscriptionModelTest(TestCase):
    """Test UserSubscription model functionality."""
    
    def setUp(self):
        """Set up test data."""
        self.user = User.objects.create_user(
            email='test@example.com',
            password='testpass123'
        )
        
        # Create a test Stripe product
        self.stripe_product = StripeProduct.objects.create(
            name='Basic Plan',
            description='Basic monthly plan',
            price=Decimal('29.99'),
            currency='USD',
            interval='month',
            credit_amount=1000,
            active=True,
            stripe_id='prod_test123',
            stripe_price_id='price_test123'
        )
    
    def test_create_subscription(self):
        """Test creating a user subscription."""
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            stripe_product_id=self.stripe_product.stripe_id,
            status='active',
            current_period_start=timezone.now(),
            current_period_end=timezone.now() + timedelta(days=30)
        )
        
        self.assertEqual(subscription.user, self.user)
        self.assertEqual(subscription.stripe_subscription_id, 'sub_test123')
        self.assertEqual(subscription.status, 'active')
        self.assertTrue(subscription.is_active)
    
    def test_subscription_str_representation(self):
        """Test string representation of subscription."""
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            status='active'
        )
        
        expected = f"{self.user.email} - Active"
        self.assertEqual(str(subscription), expected)
    
    def test_days_until_renewal(self):
        """Test days until renewal calculation."""
        future_date = timezone.now() + timedelta(days=15)
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            status='active',
            current_period_end=future_date
        )
        
        days = subscription.days_until_renewal
        # Allow for small timing differences in test execution
        self.assertIn(days, [14, 15])
    
    def test_get_stripe_product(self):
        """Test getting associated Stripe product."""
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            stripe_product_id=self.stripe_product.stripe_id,
            status='active'
        )
        
        product = subscription.get_stripe_product()
        self.assertEqual(product, self.stripe_product)
    
    def test_allocate_monthly_credits(self):
        """Test monthly credit allocation."""
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            stripe_product_id=self.stripe_product.stripe_id,
            status='active',
            current_period_start=timezone.now(),
            current_period_end=timezone.now() + timedelta(days=30)
        )
        
        # Allocate credits
        transaction = subscription.allocate_monthly_credits()
        
        self.assertIsNotNone(transaction)
        self.assertEqual(transaction.amount, Decimal('1000'))
        self.assertEqual(transaction.credit_type, 'SUBSCRIPTION')
        self.assertEqual(transaction.user, self.user)


@override_settings(
    DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}},
    STRIPE_ENABLED=False,
    STRIPE_LIVE_MODE=False,
)
class SubscriptionViewsTest(TestCase):
    """Test subscription-related views."""
    
    def setUp(self):
        """Set up test data."""
        self.client = Client()
        self.user = User.objects.create_user(
            email='test@example.com',
            password='testpass123'
        )
        
        # Create test Stripe products
        self.basic_plan = StripeProduct.objects.create(
            name='Basic Plan',
            description='Basic monthly plan',
            price=Decimal('29.99'),
            currency='USD',
            interval='month',
            credit_amount=1000,
            active=True,
            stripe_id='prod_basic',
            stripe_price_id='price_basic'
        )
        
        self.pro_plan = StripeProduct.objects.create(
            name='Pro Plan',
            description='Pro monthly plan',
            price=Decimal('49.99'),
            currency='USD',
            interval='month',
            credit_amount=2000,
            active=True,
            stripe_id='prod_pro',
            stripe_price_id='price_pro'
        )
    
    def test_subscription_page_requires_login(self):
        """Test that subscription page requires authentication."""
        response = self.client.get(reverse('admin_dashboard:subscription'))
        self.assertEqual(response.status_code, 302)  # Redirect to login
    
    @patch('admin_dashboard.views.stripe_enabled', True)
    @patch('admin_dashboard.views.STRIPE_AVAILABLE', True)
    @patch('admin_dashboard.views.is_feature_enabled')
    def test_subscription_page_authenticated(self, mock_is_feature_enabled):
        """Test subscription page for authenticated user."""
        mock_is_feature_enabled.return_value = True
        self.client.login(email='test@example.com', password='testpass123')
        response = self.client.get(reverse('admin_dashboard:subscription'))
        
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Subscription Management')
        self.assertContains(response, 'Basic Plan')
        self.assertContains(response, 'Pro Plan')
    
    @patch('admin_dashboard.views.stripe_enabled', True)
    @patch('admin_dashboard.views.STRIPE_AVAILABLE', True)
    @patch('admin_dashboard.views.is_feature_enabled')
    def test_subscription_page_with_active_subscription(self, mock_is_feature_enabled):
        """Test subscription page when user has active subscription."""
        mock_is_feature_enabled.return_value = True
        self.client.login(email='test@example.com', password='testpass123')
        
        # Create active subscription
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            stripe_product_id=self.basic_plan.stripe_id,
            status='active',
            current_period_start=timezone.now(),
            current_period_end=timezone.now() + timedelta(days=30)
        )
        
        response = self.client.get(reverse('admin_dashboard:subscription'))
        
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Active')
        self.assertContains(response, 'Basic Plan')
        self.assertContains(response, 'Current Plan')
    
    @patch('admin_dashboard.views.stripe_manager')
    @patch('admin_dashboard.views.is_feature_enabled')
    def test_create_subscription_checkout_success(self, mock_is_feature_enabled, mock_stripe_manager):
        """Test successful subscription checkout creation."""
        mock_is_feature_enabled.return_value = True
        self.client.login(email='test@example.com', password='testpass123')
        
        # Mock Stripe manager
        mock_stripe_manager.create_checkout_session.return_value = MagicMock(
            url='https://checkout.stripe.com/test'
        )
        
        # Create Stripe customer
        StripeCustomer.objects.create(
            user=self.user,
            stripe_id='cus_test123',
            email=self.user.email
        )
        
        response = self.client.post(
            reverse('admin_dashboard:create_subscription_checkout'),
            {'product_id': self.basic_plan.id},
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        
        # Should either succeed (200) or fail gracefully (400) in test environment
        self.assertIn(response.status_code, [200, 400])
        data = response.json()
        
        if response.status_code == 200:
            self.assertIn('checkout_url', data)
            self.assertEqual(data['checkout_url'], 'https://checkout.stripe.com/test')
        else:
            # In test environment, might fail due to Stripe configuration
            self.assertIn('error', data)
    
    def test_create_subscription_checkout_invalid_product(self):
        """Test subscription checkout with invalid product."""
        self.client.login(email='test@example.com', password='testpass123')
        
        response = self.client.post(
            reverse('admin_dashboard:create_subscription_checkout'),
            {'product_id': 99999},  # Non-existent product
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        
        # Should return an error (either 400 or 404)
        self.assertIn(response.status_code, [400, 404])
        data = response.json()
        self.assertIn('error', data)
    
    def test_subscription_success_page(self):
        """Test subscription success page."""
        self.client.login(email='test@example.com', password='testpass123')
        
        response = self.client.get(
            reverse('admin_dashboard:subscription_success'),
            {'session_id': 'cs_test123'}
        )
        
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Subscription Activated!')
    
    def test_subscription_cancel_page(self):
        """Test subscription cancel page."""
        self.client.login(email='test@example.com', password='testpass123')
        
        response = self.client.get(reverse('admin_dashboard:subscription_cancel'))
        
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Subscription Canceled')


@override_settings(
    DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}},
    STRIPE_ENABLED=False,
    STRIPE_LIVE_MODE=False,
)
class CreditBalanceBreakdownTest(TestCase):
    """Test credit balance breakdown functionality."""
    
    def setUp(self):
        """Set up test data."""
        self.user = User.objects.create_user(
            email='test@example.com',
            password='testpass123'
        )
        self.credit_account = CreditAccount.get_or_create_for_user(self.user)
    
    def test_balance_breakdown_empty(self):
        """Test balance breakdown with no transactions."""
        breakdown = self.credit_account.get_balance_by_type()
        
        self.assertEqual(breakdown['subscription'], Decimal('0.00'))
        self.assertEqual(breakdown['pay_as_you_go'], Decimal('0.00'))
        self.assertEqual(breakdown['total'], Decimal('0.00'))
    
    def test_balance_breakdown_with_transactions(self):
        """Test balance breakdown with different transaction types."""
        # Add subscription credits
        CreditTransaction.objects.create(
            user=self.user,
            amount=Decimal('1000'),
            description='Monthly subscription credits',
            credit_type='SUBSCRIPTION'
        )
        
        # Add purchase credits
        CreditTransaction.objects.create(
            user=self.user,
            amount=Decimal('500'),
            description='Credit purchase',
            credit_type='PURCHASE'
        )
        
        # Add admin credits
        CreditTransaction.objects.create(
            user=self.user,
            amount=Decimal('100'),
            description='Admin adjustment',
            credit_type='ADMIN'
        )
        
        # Consume some credits
        CreditTransaction.objects.create(
            user=self.user,
            amount=Decimal('-200'),
            description='Service usage',
            credit_type='CONSUMPTION'
        )
        
        breakdown = self.credit_account.get_balance_by_type()
        
        self.assertEqual(breakdown['subscription'], Decimal('1000'))
        self.assertEqual(breakdown['pay_as_you_go'], Decimal('600'))  # 500 + 100 (consumption is separate)
        self.assertEqual(breakdown['total'], Decimal('1600'))  # 1000 + 600 (consumption affects overall balance separately)

==> quickscale-0.27.0.tar.gz: quickscale-0.27.0/quickscale/templates/credits/tests/test_subscription.py <==
"""Tests for subscription functionality."""
from django.test import TestCase, Client, override_settings
from django.contrib.auth import get_user_model
from django.urls import reverse
from django.utils import timezone
from datetime import timedelta
from decimal import Decimal
from unittest.mock import patch, MagicMock

from credits.models import UserSubscription, CreditAccount, CreditTransaction
from stripe_manager.models import StripeProduct, StripeCustomer

User = get_user_model()


@override_settings(
    DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}},
    STRIPE_ENABLED=False,
    STRIPE_LIVE_MODE=False,
)
class UserSubscriptionModelTest(TestCase):
    """Test UserSubscription model functionality."""
    
    def setUp(self):
        """Set up test data."""
        self.user = User.objects.create_user(
            email='test@example.com',
            password='testpass123'
        )
        
        # Create a test Stripe product
        self.stripe_product = StripeProduct.objects.create(
            name='Basic Plan',
            description='Basic monthly plan',
            price=Decimal('29.99'),
            currency='USD',
            interval='month',
            credit_amount=1000,
            active=True,
            stripe_id='prod_test123',
            stripe_price_id='price_test123'
        )
    
    def test_create_subscription(self):
        """Test creating a user subscription."""
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            stripe_product_id=self.stripe_product.stripe_id,
            status='active',
            current_period_start=timezone.now(),
            current_period_end=timezone.now() + timedelta(days=30)
        )
        
        self.assertEqual(subscription.user, self.user)
        self.assertEqual(subscription.stripe_subscription_id, 'sub_test123')
        self.assertEqual(subscription.status, 'active')
        self.assertTrue(subscription.is_active)
    
    def test_subscription_str_representation(self):
        """Test string representation of subscription."""
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            status='active'
        )
        
        expected = f"{self.user.email} - Active"
        self.assertEqual(str(subscription), expected)
    
    def test_days_until_renewal(self):
        """Test days until renewal calculation."""
        future_date = timezone.now() + timedelta(days=15)
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            status='active',
            current_period_end=future_date
        )
        
        days = subscription.days_until_renewal
        # Allow for small timing differences in test execution
        self.assertIn(days, [14, 15])
    
    def test_get_stripe_product(self):
        """Test getting associated Stripe product."""
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            stripe_product_id=self.stripe_product.stripe_id,
            status='active'
        )
        
        product = subscription.get_stripe_product()
        self.assertEqual(product, self.stripe_product)
    
    def test_allocate_monthly_credits(self):
        """Test monthly credit allocation."""
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            stripe_product_id=self.stripe_product.stripe_id,
            status='active',
            current_period_start=timezone.now(),
            current_period_end=timezone.now() + timedelta(days=30)
        )
        
        # Allocate credits
        transaction = subscription.allocate_monthly_credits()
        
        self.assertIsNotNone(transaction)
        self.assertEqual(transaction.amount, Decimal('1000'))
        self.assertEqual(transaction.credit_type, 'SUBSCRIPTION')
        self.assertEqual(transaction.user, self.user)


@override_settings(
    DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}},
    STRIPE_ENABLED=False,
    STRIPE_LIVE_MODE=False,
)
class SubscriptionViewsTest(TestCase):
    """Test subscription-related views."""
    
    def setUp(self):
        """Set up test data."""
        self.client = Client()
        self.user = User.objects.create_user(
            email='test@example.com',
            password='testpass123'
        )
        
        # Create test Stripe products
        self.basic_plan = StripeProduct.objects.create(
            name='Basic Plan',
            description='Basic monthly plan',
            price=Decimal('29.99'),
            currency='USD',
            interval='month',
            credit_amount=1000,
            active=True,
            stripe_id='prod_basic',
            stripe_price_id='price_basic'
        )
        
        self.pro_plan = StripeProduct.objects.create(
            name='Pro Plan',
            description='Pro monthly plan',
            price=Decimal('49.99'),
            currency='USD',
            interval='month',
            credit_amount=2000,
            active=True,
            stripe_id='prod_pro',
            stripe_price_id='price_pro'
        )
    
    def test_subscription_page_requires_login(self):
        """Test that subscription page requires authentication."""
        response = self.client.get(reverse('admin_dashboard:subscription'))
        self.assertEqual(response.status_code, 302)  # Redirect to login
    
    @patch('admin_dashboard.views.stripe_enabled', True)
    @patch('admin_dashboard.views.STRIPE_AVAILABLE', True)
    @patch('admin_dashboard.views.is_feature_enabled')
    def test_subscription_page_authenticated(self, mock_is_feature_enabled):
        """Test subscription page for authenticated user."""
        mock_is_feature_enabled.return_value = True
        self.client.login(email='test@example.com', password='testpass123')
        response = self.client.get(reverse('admin_dashboard:subscription'))
        
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Subscription Management')
        self.assertContains(response, 'Basic Plan')
        self.assertContains(response, 'Pro Plan')
    
    @patch('admin_dashboard.views.stripe_enabled', True)
    @patch('admin_dashboard.views.STRIPE_AVAILABLE', True)
    @patch('admin_dashboard.views.is_feature_enabled')
    def test_subscription_page_with_active_subscription(self, mock_is_feature_enabled):
        """Test subscription page when user has active subscription."""
        mock_is_feature_enabled.return_value = True
        self.client.login(email='test@example.com', password='testpass123')
        
        # Create active subscription
        subscription = UserSubscription.objects.create(
            user=self.user,
            stripe_subscription_id='sub_test123',
            stripe_product_id=self.basic_plan.stripe_id,
            status='active',
            current_period_start=timezone.now(),
            current_period_end=timezone.now() + timedelta(days=30)
        )
        
        response = self.client.get(reverse('admin_dashboard:subscription'))
        
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Active')
        self.assertContains(response, 'Basic Plan')
        self.assertContains(response, 'Current Plan')
    
    @patch('admin_dashboard.views.stripe_manager')
    @patch('admin_dashboard.views.is_feature_enabled')
    def test_create_subscription_checkout_success(self, mock_is_feature_enabled, mock_stripe_manager):
        """Test successful subscription checkout creation."""
        mock_is_feature_enabled.return_value = True
        self.client.login(email='test@example.com', password='testpass123')
        
        # Mock Stripe manager
        mock_stripe_manager.create_checkout_session.return_value = MagicMock(
            url='https://checkout.stripe.com/test'
        )
        
        # Create Stripe customer
        StripeCustomer.objects.create(
            user=self.user,
            stripe_id='cus_test123',
            email=self.user.email
        )
        
        response = self.client.post(
            reverse('admin_dashboard:create_subscription_checkout'),
            {'product_id': self.basic_plan.id},
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        
        # Should either succeed (200) or fail gracefully (400) in test environment
        self.assertIn(response.status_code, [200, 400])
        data = response.json()
        
        if response.status_code == 200:
            self.assertIn('checkout_url', data)
            self.assertEqual(data['checkout_url'], 'https://checkout.stripe.com/test')
        else:
            # In test environment, might fail due to Stripe configuration
            self.assertIn('error', data)
    
    def test_create_subscription_checkout_invalid_product(self):
        """Test subscription checkout with invalid product."""
        self.client.login(email='test@example.com', password='testpass123')
        
        response = self.client.post(
            reverse('admin_dashboard:create_subscription_checkout'),
            {'product_id': 99999},  # Non-existent product
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        
        # Should return an error (either 400 or 404)
        self.assertIn(response.status_code, [400, 404])
        data = response.json()
        self.assertIn('error', data)
    
    def test_subscription_success_page(self):
        """Test subscription success page."""
        self.client.login(email='test@example.com', password='testpass123')
        
        response = self.client.get(
            reverse('admin_dashboard:subscription_success'),
            {'session_id': 'cs_test123'}
        )
        
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Subscription Activated!')
    
    def test_subscription_cancel_page(self):
        """Test subscription cancel page."""
        self.client.login(email='test@example.com', password='testpass123')
        
        response = self.client.get(reverse('admin_dashboard:subscription_cancel'))
        
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Subscription Canceled')


@override_settings(
    DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}},
    STRIPE_ENABLED=False,
    STRIPE_LIVE_MODE=False,
)
class CreditBalanceBreakdownTest(TestCase):
    """Test credit balance breakdown functionality."""
    
    def setUp(self):
        """Set up test data."""
        self.user = User.objects.create_user(
            email='test@example.com',
            password='testpass123'
        )
        self.credit_account = CreditAccount.get_or_create_for_user(self.user)
    
    def test_balance_breakdown_empty(self):
        """Test balance breakdown with no transactions."""
        breakdown = self.credit_account.get_balance_by_type_available()
        
        self.assertEqual(breakdown['subscription'], Decimal('0.00'))
        self.assertEqual(breakdown['pay_as_you_go'], Decimal('0.00'))
        self.assertEqual(breakdown['total'], Decimal('0.00'))
    
    def test_balance_breakdown_with_transactions(self):
        """Test balance breakdown with different transaction types using priority consumption logic."""
        # Add subscription credits
        CreditTransaction.objects.create(
            user=self.user,
            amount=Decimal('1000'),
            description='Monthly subscription credits',
            credit_type='SUBSCRIPTION'
        )
        
        # Add purchase credits
        CreditTransaction.objects.create(
            user=self.user,
            amount=Decimal('500'),
            description='Credit purchase',
            credit_type='PURCHASE'
        )
        
        # Add admin credits
        CreditTransaction.objects.create(
            user=self.user,
            amount=Decimal('100'),
            description='Admin adjustment',
            credit_type='ADMIN'
        )
        
        # Consume some credits
        CreditTransaction.objects.create(
            user=self.user,
            amount=Decimal('-200'),
            description='Service usage',
            credit_type='CONSUMPTION'
        )
        
        breakdown = self.credit_account.get_balance_by_type_available()
        
        # With priority consumption, consumption comes from subscription credits first
        # Subscription: 1000 - 200 = 800 remaining
        # Pay-as-you-go: 500 + 100 = 600 (untouched)
        self.assertEqual(breakdown['subscription'], Decimal('800'))  # 1000 - 200 consumption
        self.assertEqual(breakdown['pay_as_you_go'], Decimal('600'))  # 500 + 100 (not consumed yet)
        self.assertEqual(breakdown['total'], Decimal('1400'))  # 800 + 600
